@@ -14,4 +14,4 @@ class PredHandler(Callback):
     def on_pred_end(self) -> None: self.preds = torch.cat(self.preds)
     def get_preds(self) -> np.ndarray: return self.preds
     def on_forwards_end(self) -> None:
-        if self.model.fit_params.state == 'test': self.preds.append(self.model.fit_params.y_pred)
+        if self.model.fit_params.state == 'test': self.preds.append(self.model.fit_params.y_pred.cpu().detach())
